import numpy as np

from moviepy.editor import VideoFileClip, concatenate_videoclips
from moviepy.video.fx.all import speedx
from tqdm import tqdm


class Clip:
    def __init__(self, clip_path, min_loud_part_duration, silence_part_speed):
        self.clip = VideoFileClip(clip_path)
        self.audio = Audio(self.clip.audio)
        self.cut_to_method = {
            "silent": self.jumpcut_silent_parts,
            "voiced": self.jumpcut_voiced_parts,
        }
        self.min_loud_part_duration = min_loud_part_duration
        self.silence_part_speed = silence_part_speed

    def get_duration(self):
        return self.clip.duration

    def jumpcut(
        self,
        cuts,
        magnitude_threshold_ratio,
        duration_threshold_in_seconds,
        failure_tolerance_ratio,
        space_on_edges,
    ):

        intervals_to_cut = self.audio.get_intervals_to_cut(
            magnitude_threshold_ratio,
            duration_threshold_in_seconds,
            failure_tolerance_ratio,
            space_on_edges,
        )
        outputs = {}
        for cut in cuts:
            jumpcutted_clips = self.cut_to_method[cut](intervals_to_cut)
            outputs[cut] = concatenate_videoclips(jumpcutted_clips)

        return outputs

    def jumpcut_silent_parts(self, intervals_to_cut):
        jumpcutted_clips = []
        previous_stop = 0
        for start, stop in tqdm(intervals_to_cut, desc="Cutting silent intervals"):
            clip_before = self.clip.subclip(previous_stop, start)

            if clip_before.duration > self.min_loud_part_duration:
                jumpcutted_clips.append(clip_before)

            if self.silence_part_speed is not None:
                silence_clip = self.clip.subclip(start, stop)
                silence_clip = speedx(
                    silence_clip, self.silence_part_speed
                ).without_audio()
                jumpcutted_clips.append(silence_clip)

            previous_stop = stop

        last_clip = self.clip.subclip(stop, self.clip.duration)
        jumpcutted_clips.append(last_clip)
        return jumpcutted_clips

    def jumpcut_voiced_parts(self, intervals_to_cut):
        jumpcutted_clips = []
        for start, stop in tqdm(intervals_to_cut, desc="Cutting voiced intervals"):
            silence_clip = self.clip.subclip(start, stop)
            jumpcutted_clips.append(silence_clip)
        return jumpcutted_clips


class Audio:
    def __init__(self, audio):
        self.audio = audio
        self.fps = audio.fps

        self.signal = self.audio.to_soundarray()
        if len(self.signal.shape) == 1:
            self.signal = self.signal.reshape(-1, 1)

    def get_intervals_to_cut(
        self,
        magnitude_threshold_ratio,
        duration_threshold_in_seconds,
        failure_tolerance_ratio,
        space_on_edges,
    ):
        min_magnitude = min(abs(np.min(self.signal)), np.max(self.signal))
        magnitude_threshold = min_magnitude * magnitude_threshold_ratio
        failure_tolerance = self.fps * failure_tolerance_ratio
        duration_threshold = self.fps * duration_threshold_in_seconds
        silence_counter = 0
        failure_counter = 0

        intervals_to_cut = []
        absolute_signal = np.absolute(self.signal)
        for i, values in tqdm(
            enumerate(absolute_signal),
            desc="Getting silent intervals to cut",
            total=len(absolute_signal),
        ):
            silence = all([value < magnitude_threshold for value in values])
            silence_counter += silence
            failure_counter += not silence
            if failure_counter >= failure_tolerance:
                if silence_counter >= duration_threshold:
                    interval_end = (i - failure_counter) / self.fps
                    interval_start = interval_end - (silence_counter / self.fps)

                    interval_start += space_on_edges
                    interval_end -= space_on_edges

                    intervals_to_cut.append(
                        (abs(interval_start), interval_end)
                    )  # in seconds
                silence_counter = 0
                failure_counter = 0
        return intervals_to_cut



import argparse

from pathlib import Path

from jumpcutter.clip import Clip


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "--input", "-i", help="Path to the input video", type=Path, required=True
    )
    parser.add_argument(
        "--output",
        "-o",
        help="Path to where you want to save the output video. "
        "Note: Not all extensions are supported. Checkout moviepy's documentation",
        type=Path,
        required=True,
    )
    parser.add_argument(
        "--cut",
        "-c",
        choices=["silent", "voiced", "both"],
        help="Decides whether to cut silent parts, voiced parts, or both (creating 2 separate videos)",
        default="silent",
    )
    parser.add_argument(
        "--magnitude-threshold-ratio",
        "-m",
        help="Audio signal's values ({x in R: -1 <= x <= 1}) that are: "
        "min_magnitude = min(abs(min(x)), max(x))"
        "magnitude_threshold = min_magnitude * magnitude_threshold_ratio"
        "abs(x) < magnitude_threshold"
        "will be threated as silence audio signal values.",
        type=float,
        default=0.02,
    )
    parser.add_argument(
        "--duration-threshold",
        "-d",
        help="Minimum number of required seconds in silence to cut it out",
        type=float,
        default=0.5,
    )
    parser.add_argument(
        "--failure-tolerance-ratio",
        "-f",
        help="Consequent x values are taken into account to find silence parts of the signal. "
        "Failure tolerance ratio leaves room for some error. For example if failure threshold "
        "ratio is 0.1, then in 1 second silence signal, it is allowed to have 0.1 seconds "
        "non silence signal, and it is still going to be treated as a silence signal.",
        type=float,
        default=0.1,
    )
    parser.add_argument(
        "--space-on-edges",
        "-s",
        help="Leaves some space on the edges of silence cut. E.g. if it is found that there is silence "
        "between 10th and 20th second of the video, then instead of cutting it out directly, "
        "we cut out  (10+space_on_edges)th and (20-space_on_edges)th seconds of the clip",
        type=float,
        default=0.1,
    )
    parser.add_argument(
        "--silence-part-speed",
        "-x",
        help="If this parameter is set, it will speed up the silence parts x times instead of cutting them out",
        type=int,
        required=False,
    )
    parser.add_argument(
        "--min-loud-part-duration",
        "-l",
        help="If this parameter is set, loud parts of the clip that are shorter than this parameter "
        "(in seconds) will also be cutted",
        type=int,
        required=False,
        default=-1,
    )
    parser.add_argument(
        "--codec",
        help="Codec to use for image encoding. Can be any codec supported by ffmpeg. If the filename "
        "has extension ‘.mp4’, ‘.ogv’, ‘.webm’, the codec will be set accordingly, but you can still set "
        "it if you don’t like the default. For other extensions, the output filename must be set accordingly.",
        type=str,
        required=False,
    )
    parser.add_argument(
        "--bitrate",
        help="Desired bitrate for the output video",
        type=int,
        required=False,
    )

    args = parser.parse_args()
    print("Running with the arguments:")
    print(args)
    print(60 * "-")
    if args.duration_threshold / 2 <= args.space_on_edges:
        print(60 * "*")
        print("WARNING:")
        print(
            "You have selected space_on_edges >= duration_threshold/2. This may cause overlapping sequences"
        )
        print(60 * "*")

    input_path = args.input.resolve()
    output_path = args.output.resolve()
    cuts = [args.cut] if args.cut != "both" else ["silent", "voiced"]
    codec = args.codec
    bitrate = args.bitrate

    clip = Clip(str(input_path), args.min_loud_part_duration, args.silence_part_speed)
    outputs = clip.jumpcut(
        cuts,
        args.magnitude_threshold_ratio,
        args.duration_threshold,
        args.failure_tolerance_ratio,
        args.space_on_edges,
    )
    for cut_type, jumpcutted_clip in outputs.items():
        if len(outputs) == 2:
            jumpcutted_clip.write_videofile(
                str(
                    output_path.parent
                    / f"{output_path.stem}_{cut_type}_parts_cutted{output_path.suffix}"
                ),
                codec=codec,
                bitrate=bitrate,
            )
        else:
            jumpcutted_clip.write_videofile(
                str(output_path), codec=codec, bitrate=bitrate
            )


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['jumpcutter']

package_data = \
{'': ['*']}

install_requires = \
['moviepy>=1.0.3,<1.1.0', 'tqdm>=4.60.0,<4.61.0']

entry_points = \
{'console_scripts': ['jumpcutter = jumpcutter.__main__:main']}

setup_kwargs = {
    'name': 'jumpcutter',
    'version': '0.1.5',
    'description': 'Jumpcut silent parts of your videos automagically',
    'long_description': '## What is jumpcutter?\n\nJumpcutter is a program that is written in Python to automatically jump-cut silent parts of your videos.\nThe purpose here is to ease your post recording work.\n\nCheck out [the medium post](https://medium.com/@emkademy/how-to-jump-cut-silent-parts-of-your-videos-automatically-with-python-2e4b96320dc1)\nfor more information.\n\n## Installation\nYou can install jumpcutter by simply:\n\n```bash\npip install jumpcutter \n```\n\n## Demo\n\n[![Watch the video](https://img.youtube.com/vi/UDjzm_lzWOA/hqdefault.jpg)](https://youtu.be/UDjzm_lzWOA)\n\n## How to run it?\nThere are 11 command line arguments you can run the program with. \nBefore explaining them, I would like to say that most of these parameters \nhave a default value that “just works”. So, if you don’t want you don’t need to specify \n(or know) almost any of these parameters. You will be just fine with the default values.\n\n1. `--input`, `-i`: Path to the video that you want to jump-cut.\n2. `--output`, `-o`: Path to where you want to save the output video.\n3. `--magnitude-threshold-ratio`, `-m`: The percentage of the maximum value of your audio signal that you would like to \n     consider as silent a signal (default: 0.02).\n4. `--duration-threshold`, `-d`: Minimum number of required seconds in silence to cut it out. For example if this parameter \n     is 0.5, it means that the silence parts have to last minimum 0.5 seconds, otherwise they won\'t be jump-cut (default: 0.5).\n5. `--failure-tolerance-ratio`, `-f`: Most of the times, there are 44100 audio signal values in 1 second of a video. \n     Let\'s say the "--duration-threshold" was set to 0.5. This means that, we need to check minimum 22050 signal \n     values to see if there is a silent part of not. What happens if we found 22049 values that we consider as silent, \n     but there is 1 value that is above our threshold. Should we just throw this part of the video and consider it as a \n     loud signal? I think we shouldn\'t. This parameter leaves some room for failure, it tolerates high signal values until \n     some point. Let\'s say it is set to 0.1, it means that 10% of the signal that is currently being investigated can \n     have values that are higher than our threshold, but they are still going to be considered as a silent part (default: 0.1).\n6. `--spaces-on-edges`, `-s`: Leaves some space on the edges of silence cut. E.g. if it is found that there is \n     silence between 10th and 20th second of the video, then instead of cutting it out directly, we cut out \n     (10+space_on_edges)th and (20-space_on_edges)th seconds of the clip (default: 0.1).\n7. `--silence-part-speed`, `-x`: If this parameter is given, instead of cutting the silent parts out, the script will \n     speed them up "x" times.\n8. `--min-loud-part-duration`, `-l`: If this parameter is given, loud parts of the video that are shorter then this \n     parameter will also be cut.\n9. `--cut`, `-c`: If you want, you can also cut voiced parts of the video (to have some fun :)). There are 3 choices \n     you can make for this parameter: `silent`, `voiced`, `both`. If you choose `silent`, silent parts of the video will\n     be cutted; if you choose `voiced`, voiced parts of the video will be cutted; if you choose `both` 2 videos will be\n     saved: 1 for the silent parts, 1 for the voiced parts (default: silent).\n10. `--codec`: Codec to use for image encoding. Can be any codec supported by ffmpeg. If the filename \n     has extension ‘.mp4’, ‘.ogv’, ‘.webm’, the codec will be set accordingly, but you can still set\n     it if you don’t like the default. For other extensions, the output filename must be set accordingly. \n     Check [here](https://zulko.github.io/moviepy/ref/VideoClip/VideoClip.html#moviepy.video.compositing.CompositeVideoClip.CompositeVideoClip.write_videofile)\n11. `bitrate`: Desired bitrate of the output video. Leave blank if you don\'t know what this is.\n     \n## Examples of running the program\n\n```bash\n# The simplest way you can run the program\njumpcutter -i input_video.mp4 -o output_video.mp4\n# If you want, you can also set the other parameters that was mentioned\njumpcutter -i input_video.mp4 -o output_video.mp4 -m 0.05 -d 1.0 -f 0.2 -s 0.2 -x 2000 -l 1.0 -c both\n```\n',
    'author': 'Kivanc Yuksel',
    'author_email': 'emkademy@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kivancyuksel/jumpcutter.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
